@@ -57,6 +57,14 @@ AudioDeviceID			adid;
  * @brief The stream format used by the CoreAudio device.
  */
 AudioStreamBasicDescription	afmt;
+#ifdef MAC_OS_X_VERSION_10_5
+/**
+ * @brief The IO-process ID.
+ */
+AudioDeviceIOProcID		aprocid;
+#else /* !MAC_OS_X_VERSION_10_5 */
+#define aprocid			audio_output_ioproc
+#endif /* MAC_OS_X_VERSION_10_5 */
 
 /**
  * @brief The buffer that will be played when the current buffer is
@@ -86,6 +94,13 @@ GMutex				*abuflock;
  *        application that a buffer has been processed.
  */
 GCond				*abufdrained;
+#ifdef BUILD_VOLUME
+/**
+ * @brief Preferred audio channels used for audio playback. We use it to
+ *        set the volume.
+ */
+UInt32				achans[2] = { 0, 0 };
+#endif /* BUILD_VOLUME */
 
 /**
  * @brief Pull-function needed by CoreAudio to copy data to the audio
@@ -103,7 +118,7 @@ audio_output_ioproc(AudioDeviceID inDevice, const AudioTimeStamp *inNow,
 	/* Stop the IOProc handling if we're going idle */
 	len = g_atomic_int_get(&abufulen);
 	if (len == 0)
-		AudioDeviceStop(adid, audio_output_ioproc);
+		AudioDeviceStop(adid, aprocid);
 
 	/* Convert the data to floats */
 	for (i = 0; i < len; i++)
@@ -154,6 +169,13 @@ audio_output_open(void)
 	    kAudioDevicePropertyBufferSize, &size, &abuflen) != 0)
 		goto error;
 
+#ifdef BUILD_VOLUME
+	/* Store the audio channels */
+	size = sizeof achans;
+	AudioDeviceGetProperty(adid, 0, false,
+	    kAudioDevicePropertyPreferredChannelsForStereo, &size, &achans);
+#endif /* BUILD_VOLUME */
+
 	/* The buffer size reported is in floats */
 	abuflen /= sizeof(float);
 	abufnew = g_malloc(abuflen * sizeof(int16_t));
@@ -164,12 +186,17 @@ audio_output_open(void)
 	abufdrained = g_cond_new();
 
 	/* Add our own I/O handling routine */
+#ifdef MAC_OS_X_VERSION_10_5
+	if (AudioDeviceCreateIOProcID(adid, audio_output_ioproc, NULL,
+	    &aprocid) != 0)
+#else /* !MAC_OS_X_VERSION_10_5 */
 	if (AudioDeviceAddIOProc(adid, audio_output_ioproc, NULL) != 0)
+#endif /* MAC_OS_X_VERSION_10_5 */
 		goto error;
 
 	return (0);
 error:
-	g_printerr(_("Cannot open the audio device.\n"));
+	g_printerr("%s\n", _("Cannot open the audio device."));
 	return (-1);
 }
 
@@ -216,7 +243,7 @@ audio_output_play(struct audio_file *fd)
 	g_atomic_int_set(&abufulen, len);
 
 	/* Start processing of the data */
-	AudioDeviceStart(adid, audio_output_ioproc);
+	AudioDeviceStart(adid, aprocid);
 
 	return (0);
 }
@@ -224,9 +251,60 @@ audio_output_play(struct audio_file *fd)
 void
 audio_output_close(void)
 {
-	AudioDeviceStop(adid, audio_output_ioproc);
-	AudioDeviceRemoveIOProc(adid, audio_output_ioproc);
+	AudioDeviceStop(adid, aprocid);
+#ifdef MAC_OS_X_VERSION_10_5
+	AudioDeviceDestroyIOProcID(adid, aprocid);
+#else /* !MAC_OS_X_VERSION_10_5 */
+	AudioDeviceRemoveIOProc(adid, aprocid);
+#endif /* MAC_OS_X_VERSION_10_5 */
 
 	g_free(abufnew);
 	g_free(abufcur);
 }
+
+#ifdef BUILD_VOLUME
+/**
+ * @brief Adjust the audio output by a certain ratio and return the new
+ *        value.
+ */
+static int
+audio_output_volume_adjust(Float32 n)
+{
+	Float32 vl, vr, vn;
+	UInt32 size = sizeof vl;
+
+	/*
+	 * Merge left and right. On Mac OS X, we want to do this. My
+	 * PowerBook has this odd bug that the built-in OS X volume
+	 * applet makes the sound card go unbalanced after a long amount
+	 * of time. We can prevent that over here...
+	 */
+	if (AudioDeviceGetProperty(adid, achans[0], false,
+	    kAudioDevicePropertyVolumeScalar, &size, &vl) != 0 ||
+	    AudioDeviceGetProperty(adid, achans[1], false,
+	    kAudioDevicePropertyVolumeScalar, &size, &vr) != 0)
+		return (-1);
+	vn = CLAMP((vl + vr) / 2.0 + n, 0.0, 1.0);
+
+	/* Set the new volume */
+	if (AudioDeviceSetProperty(adid, 0, achans[0], false,
+	    kAudioDevicePropertyVolumeScalar, size, &vn) != 0 ||
+	    AudioDeviceSetProperty(adid, 0, achans[1], false,
+	    kAudioDevicePropertyVolumeScalar, size, &vn) != 0)
+		return (-1);
+
+	return (vn * 100.0);
+}
+
+int
+audio_output_volume_up(void)
+{
+	return audio_output_volume_adjust(0.04);
+}
+
+int
+audio_output_volume_down(void)
+{
+	return audio_output_volume_adjust(-0.04);
+}
+#endif /* BUILD_VOLUME */
