@@ -42,6 +42,12 @@
  * @brief File descriptor of the audio device handle.
  */
 static int dev_fd;
+#ifdef BUILD_VOLUME
+/**
+ * @brief File descriptor of the mixer device.
+ */
+static int mix_fd;
+#endif /* BUILD_VOLUME */
 /**
  * @brief Sample rate of the audio device handle.
  */
@@ -56,12 +62,19 @@ audio_output_open(void)
 {
 	const char *dev;
 
+	/* Open the audio device */
 	dev = config_getopt("audio.output.oss.device");
 	if ((dev_fd = open(dev, O_WRONLY)) == -1) {
 		g_printerr(_("Cannot open audio device \"%s\".\n"), dev);
 		return (-1);
 	}
 
+#ifdef BUILD_VOLUME
+	/* Open the mixer device */
+	dev = config_getopt("audio.output.oss.mixer");
+	mix_fd = open(dev, O_RDWR);
+#endif /* BUILD_VOLUME */
+
 	return (0);
 }
 
@@ -114,3 +127,44 @@ audio_output_close(void)
 {
 	close(dev_fd);
 }
+
+#ifdef BUILD_VOLUME
+/**
+ * @brief Adjust the audio output by a certain percentage and return the
+ *        new value.
+ */
+static int
+audio_output_volume_adjust(int n)
+{
+	int vol, out;
+
+	/* We can't use the mixer */
+	if (mix_fd < 0)
+		return (-1);
+
+	if (ioctl(mix_fd, MIXER_READ(SOUND_MIXER_VOLUME), &vol) == -1)
+		return (-1);
+	
+	/* XXX: Merge left and right */
+	vol = ((vol & 0x7f) + ((vol >> 8) & 0x7f)) / 2;
+	vol = CLAMP(vol + n, 0, 100);
+	out = (vol << 8) | vol;
+
+	if (ioctl(mix_fd, MIXER_WRITE(SOUND_MIXER_VOLUME), &out) == -1)
+		return (-1);
+
+	return (vol);
+}
+
+int
+audio_output_volume_up(void)
+{
+	return audio_output_volume_adjust(4);
+}
+
+int
+audio_output_volume_down(void)
+{
+	return audio_output_volume_adjust(-4);
+}
+#endif /* BUILD_VOLUME */
