@@ -141,6 +141,9 @@ static struct config_entry configlist[] = {
 #endif /* BUILD_AO */
 #ifdef BUILD_OSS
 	{ "audio.output.oss.device",	OSS_DEVICE,	NULL,		NULL },
+#ifdef BUILD_VOLUME
+	{ "audio.output.oss.mixer",	"/dev/mixer",	NULL,		NULL },
+#endif /* BUILD_VOLUME */
 #endif /* BUILD_OSS */
 	{ "gui.browser.defaultpath",	"",		NULL,		NULL },
 	{ "gui.color.bar.bg",		"blue",		valid_color,	NULL },
