@@ -67,36 +67,54 @@ static regex_t cursearchregex;
  * @brief The last seek string that has been entered.
  */
 static char *curseek = NULL;
+/**
+ * @brief Determine if we're already trying to shut down, to prevent
+ *        multiple signals being handled while quitting.
+ */
+static int shutting_down = 0;
+/**
+ * @brief Add the Ctrl modifier to a character.
+ */
+#define CTRL(x) (((x) - 'A' + 1) & 0x7f)
 
 static void gui_input_search(void);
 
 /**
  * @brief Fetch a character from the keyboard, already processing
- *        terminal resizes.
+ *        terminal resizes and awkward bugs.
  */
 static int
 gui_input_getch(void)
 {
 	int ch;
 
-	do {
+	for (;;) {
 		ch = getch();
-		/* Redraw everything when we get KEY_RESIZE or ^L */
-		if (ch == KEY_RESIZE || ch == 0x0c) {
+
+		switch (ch) {
+		/* Redraw everything when resizing */
+		case KEY_RESIZE:
+		case CTRL('L'):
 			gui_draw_resize();
-			ch = ERR;
+			continue;
+
+		/* Catch backspace button */
+		case CTRL('H'):
+		case CTRL('?'):
+			return (KEY_BACKSPACE);
+
+		/* FreeBSD returns KEY_SELECT instead of KEY_END */
+		case KEY_SELECT:
+			return (KEY_END);
+
+		/* Try again */
+		case ERR:
+			continue;
 		}
-	} while (ch == ERR);
 
-	/* End-key on xBSD is select? */
-	if (ch == KEY_SELECT)
-		return (KEY_END);
+		break;
+	}
 
-	/* ^H or ^? backspace */
-	if (ch == 0x08 || ch == 0x7f)
-		return (KEY_BACKSPACE);
-	
-	/* Valid character */
 	return (ch);
 }
 
@@ -207,6 +225,8 @@ gui_input_search(void)
 static void
 gui_input_quit(void)
 {
+	shutting_down = 1;
+
 	playq_shutdown();
 #ifdef BUILD_SCROBBLER
 	scrobbler_shutdown();
@@ -352,31 +372,37 @@ struct gui_binding {
  */
 static struct gui_binding kbdbindings[] = {
 	/* Application-wide keyboard bindings */
-	{ -1, '<',			gui_input_cursong_seek_backward, },
-	{ -1, '>',			gui_input_cursong_seek_forward, },
+#ifdef BUILD_VOLUME
+	{ -1, '(',			gui_playq_volume_down },
+	{ -1, ')',			gui_playq_volume_up },
+#endif /* BUILD_VOLUME */
+	{ -1, '<',			gui_input_cursong_seek_backward },
+	{ -1, '>',			gui_input_cursong_seek_forward },
 	{ -1, 'a',			gui_browser_playq_add_after },
 	{ -1, 'A',			gui_browser_playq_add_tail },
 	{ -1, 'b',			playq_cursong_next },
-	{ -1, 'c',			playq_cursong_pause, },
+	{ -1, 'c',			playq_cursong_pause },
 	{ -1, 'C',			gui_browser_chdir },
 	{ -1, 'd',			gui_playq_song_remove },
 	{ -1, 'D',			gui_playq_song_remove_all },
 	{ -1, 'h',			gui_browser_dir_parent },
 	{ -1, 'i',			gui_browser_playq_add_before },
 	{ -1, 'I',			gui_browser_playq_add_head },
-	{ -1, 'J',			gui_input_cursong_seek_jump, }, /* ^J */
+	{ -1, 'J',			gui_input_cursong_seek_jump },
 	{ -1, 'l',			gui_browser_dir_enter },
 	{ -1, 'q',			gui_input_askquit },
 	{ -1, 'r',			playq_repeat_toggle },
 	{ -1, 'R',			gui_playq_song_randomize },
-	{ -1, 'v',			playq_cursong_stop, },
+	{ -1, 'v',			playq_cursong_stop },
 	{ -1, 'w',			gui_browser_write_playlist },
 	{ -1, 'x',			gui_playq_song_select },
 	{ -1, 'z',			playq_cursong_prev },
-	{ -1, '[',			gui_playq_song_moveup },
-	{ -1, ']',			gui_playq_song_movedown },
+	{ -1, '[',			gui_playq_song_move_up },
+	{ -1, ']',			gui_playq_song_move_down },
+	{ -1, '{',			gui_playq_song_move_head },
+	{ -1, '}',			gui_playq_song_move_tail },
 	{ -1, '\t', 			gui_input_switchfocus },
-	{ -1, 0x17, 			gui_input_switchfocus }, /* ^W */
+	{ -1, CTRL('W'),		gui_input_switchfocus },
 	{ -1, '/',			gui_input_search },
 	{ -1, 'n',			gui_input_searchnext },
 	{ -1, KEY_LEFT,			gui_browser_dir_parent },
@@ -385,15 +411,15 @@ static struct gui_binding kbdbindings[] = {
 	/* Keyboard bindings for the file browser */
 	{ GUI_FOCUS_BROWSER, ' ',	gui_browser_cursor_pagedown },
 	{ GUI_FOCUS_BROWSER, 'f',	gui_browser_fullpath },
-	{ GUI_FOCUS_BROWSER, 'G',	gui_browser_cursor_bottom },
-	{ GUI_FOCUS_BROWSER, 'g',	gui_browser_cursor_top },
+	{ GUI_FOCUS_BROWSER, 'G',	gui_browser_cursor_tail },
+	{ GUI_FOCUS_BROWSER, 'g',	gui_browser_cursor_head },
 	{ GUI_FOCUS_BROWSER, 'j',	gui_browser_cursor_down },
 	{ GUI_FOCUS_BROWSER, 'k',	gui_browser_cursor_up },
-	{ GUI_FOCUS_BROWSER, 0x02,	gui_browser_cursor_pageup }, /* ^B */
-	{ GUI_FOCUS_BROWSER, 0x06,	gui_browser_cursor_pagedown }, /* ^F */
+	{ GUI_FOCUS_BROWSER, CTRL('B'), gui_browser_cursor_pageup },
+	{ GUI_FOCUS_BROWSER, CTRL('F'), gui_browser_cursor_pagedown },
 	{ GUI_FOCUS_BROWSER, KEY_DOWN,	gui_browser_cursor_down },
-	{ GUI_FOCUS_BROWSER, KEY_END,	gui_browser_cursor_bottom },
-	{ GUI_FOCUS_BROWSER, KEY_HOME,	gui_browser_cursor_top },
+	{ GUI_FOCUS_BROWSER, KEY_END,	gui_browser_cursor_tail },
+	{ GUI_FOCUS_BROWSER, KEY_HOME,	gui_browser_cursor_head },
 	{ GUI_FOCUS_BROWSER, KEY_NPAGE,	gui_browser_cursor_pagedown },
 	{ GUI_FOCUS_BROWSER, KEY_PPAGE,	gui_browser_cursor_pageup },
 	{ GUI_FOCUS_BROWSER, KEY_UP,	gui_browser_cursor_up },
@@ -401,15 +427,15 @@ static struct gui_binding kbdbindings[] = {
 	/* Keyboard bindings for the playlist */
 	{ GUI_FOCUS_PLAYQ, ' ',		gui_playq_cursor_pagedown },
 	{ GUI_FOCUS_PLAYQ, 'f',		gui_playq_fullpath },
-	{ GUI_FOCUS_PLAYQ, 'G',		gui_playq_cursor_bottom },
-	{ GUI_FOCUS_PLAYQ, 'g',		gui_playq_cursor_top },
+	{ GUI_FOCUS_PLAYQ, 'G',		gui_playq_cursor_tail },
+	{ GUI_FOCUS_PLAYQ, 'g',		gui_playq_cursor_head },
 	{ GUI_FOCUS_PLAYQ, 'j',		gui_playq_cursor_down },
 	{ GUI_FOCUS_PLAYQ, 'k',		gui_playq_cursor_up },
-	{ GUI_FOCUS_PLAYQ, 0x02,	gui_playq_cursor_pageup }, /* ^B */
-	{ GUI_FOCUS_PLAYQ, 0x06,	gui_playq_cursor_pagedown }, /* ^F */
+	{ GUI_FOCUS_PLAYQ, CTRL('B'),	gui_playq_cursor_pageup },
+	{ GUI_FOCUS_PLAYQ, CTRL('F'),	gui_playq_cursor_pagedown },
 	{ GUI_FOCUS_PLAYQ, KEY_DOWN,	gui_playq_cursor_down },
-	{ GUI_FOCUS_PLAYQ, KEY_END,	gui_playq_cursor_bottom },
-	{ GUI_FOCUS_PLAYQ, KEY_HOME,	gui_playq_cursor_top },
+	{ GUI_FOCUS_PLAYQ, KEY_END,	gui_playq_cursor_tail },
+	{ GUI_FOCUS_PLAYQ, KEY_HOME,	gui_playq_cursor_head },
 	{ GUI_FOCUS_PLAYQ, KEY_NPAGE,	gui_playq_cursor_pagedown },
 	{ GUI_FOCUS_PLAYQ, KEY_PPAGE,	gui_playq_cursor_pageup },
 	{ GUI_FOCUS_PLAYQ, KEY_UP,	gui_playq_cursor_up },
@@ -447,6 +473,9 @@ gui_input_sigmask(void)
 static void
 gui_input_sighandler(int signal)
 {
+	if (shutting_down)
+		return;
+
 	switch (signal) {
 	case SIGUSR1:
 		playq_cursong_pause();
@@ -460,7 +489,7 @@ gui_input_sighandler(int signal)
 	case SIGQUIT:
 	case SIGTERM:
 		gui_input_quit();
-		/* NOTREACHED */
+		g_assert_not_reached();
 	}
 }
 #endif /* G_OS_UNIX */
@@ -538,7 +567,7 @@ gui_input_askyesno(const char *question)
 			goto done;
 		case 'n':
 		case 'N':
-		case 0x03: /* ^C */
+		case CTRL('C'):
 			ret = -1;
 			goto done;
 		}
@@ -613,14 +642,14 @@ gui_input_askstring(const char *question, const char *defstr,
 				g_string_truncate(msg, msg->len - 1);
 			}
 			break;
-		case 0x03: /* ^C */
+		case CTRL('C'):
 			/* Just empty the return */
 			g_string_truncate(msg, origlen);
 			goto done;
-		case 0x15: /* ^U */
+		case CTRL('U'):
 			g_string_truncate(msg, origlen);
 			break;
-		case 0x17: /* ^W */
+		case CTRL('W'):
 			clearfirst = 0;
 			newlen = gui_input_trimword(msg);
 			g_string_truncate(msg, MAX(newlen, origlen));
