@@ -50,21 +50,21 @@
  * @brief Flag indicating if the AudioScrobbler thread has already been
  *        launched, causing the submission queue to be filled.
  */
-static char scrobbler_enabled = 0;
+static char	scrobbler_enabled = 0;
 
 /**
  * @brief Lock used to provide safe access to the AudioScrobbler queue.
  */
-static GMutex 			*scrobbler_lock;
+static GMutex 	*scrobbler_lock;
 /**
  * @brief Conditional variable used to notify the avaiability of new
  *        tracks ready for submission to AudioScrobbler.
  */
-static GCond			*scrobbler_avail;
+static GCond	*scrobbler_avail;
 /**
  * @brief Reference to AudioScrobbler submission thread.
  */
-static GThread			*scrobbler_runner;
+static GThread	*scrobbler_runner;
 
 /**
  * @brief An entry in the AudioScrobbler queue, ready to be submitted to
@@ -287,16 +287,15 @@ scrobbler_hash(time_t t, char out[32])
 {
 	char tstr[16];
 	unsigned char bin_res[16];
-	MD5_CTX ctx;
+	struct md5_context ctx = MD5CONTEXT_INITIALIZER;
 
 	/*
 	 * Generate the new MD5 value
 	 */
-	MD5Init(&ctx);
-	MD5Update(&ctx, config_getopt("scrobbler.password"), 32);
+	md5_update(&ctx, config_getopt("scrobbler.password"), 32);
 	sprintf(tstr, "%u", (unsigned int)t);
-	MD5Update(&ctx, tstr, strlen(tstr));
-	MD5Final(bin_res, &ctx);
+	md5_update(&ctx, tstr, strlen(tstr));
+	md5_final(&ctx, bin_res);
 
 	/*
 	 * Convert the result back to hexadecimal string
@@ -526,7 +525,8 @@ scrobbler_runner_thread(void *unused)
 		g_usleep(interval * 1000000);
 	}
 	
-	/* NOTREACHED */
+	g_assert_not_reached();
+	return (NULL);
 }
 
 void
